@@ -54,10 +54,17 @@ NOVA_CLIENT_VERSION = "2"
 
 OPENSTACK_RELEASES_PAIRS = [
     'trusty_icehouse', 'trusty_kilo', 'trusty_liberty',
-    'trusty_mitaka', 'xenial_mitaka', 'xenial_newton',
-    'yakkety_newton', 'xenial_ocata', 'zesty_ocata',
-    'xenial_pike', 'artful_pike', 'xenial_queens',
-    'bionic_queens', 'bionic_rocky', 'cosmic_rocky']
+    'trusty_mitaka', 'xenial_mitaka',
+    'xenial_newton', 'yakkety_newton',
+    'xenial_ocata', 'zesty_ocata',
+    'xenial_pike', 'artful_pike',
+    'xenial_queens', 'bionic_queens',
+    'bionic_rocky', 'cosmic_rocky',
+    'bionic_stein', 'disco_stein',
+    'bionic_train', 'eoan_train',
+    'bionic_ussuri', 'focal_ussuri',
+    'focal_victoria', 'groovy_victoria',
+]
 
 
 class OpenStackAmuletUtils(AmuletUtils):
@@ -87,14 +94,14 @@ class OpenStackAmuletUtils(AmuletUtils):
         validation_function = self.validate_v2_endpoint_data
         xenial_queens = OPENSTACK_RELEASES_PAIRS.index('xenial_queens')
         if openstack_release and openstack_release >= xenial_queens:
-                validation_function = self.validate_v3_endpoint_data
-                expected = {
-                    'id': expected['id'],
-                    'region': expected['region'],
-                    'region_id': 'RegionOne',
-                    'url': self.valid_url,
-                    'interface': self.not_null,
-                    'service_id': expected['service_id']}
+            validation_function = self.validate_v3_endpoint_data
+            expected = {
+                'id': expected['id'],
+                'region': expected['region'],
+                'region_id': 'RegionOne',
+                'url': self.valid_url,
+                'interface': self.not_null,
+                'service_id': expected['service_id']}
         return validation_function(endpoints, admin_port, internal_port,
                                    public_port, expected)
 
@@ -704,8 +711,8 @@ class OpenStackAmuletUtils(AmuletUtils):
                        '{}...'.format(image_name, image_url))
 
         # Download image
-        http_proxy = os.getenv('AMULET_HTTP_PROXY')
-        self.log.debug('AMULET_HTTP_PROXY: {}'.format(http_proxy))
+        http_proxy = os.getenv('OS_TEST_HTTP_PROXY')
+        self.log.debug('OS_TEST_HTTP_PROXY: {}'.format(http_proxy))
         if http_proxy:
             proxies = {'http': http_proxy}
             opener = urllib.FancyURLopener(proxies)
@@ -795,8 +802,8 @@ class OpenStackAmuletUtils(AmuletUtils):
                        '({})...'.format(image_name))
 
         # Get cirros image URL
-        http_proxy = os.getenv('AMULET_HTTP_PROXY')
-        self.log.debug('AMULET_HTTP_PROXY: {}'.format(http_proxy))
+        http_proxy = os.getenv('OS_TEST_HTTP_PROXY')
+        self.log.debug('OS_TEST_HTTP_PROXY: {}'.format(http_proxy))
         if http_proxy:
             proxies = {'http': http_proxy}
             opener = urllib.FancyURLopener(proxies)
