@@ -138,7 +138,7 @@ CEPH_CLIENT_RELATION = {
             'private-address': '10.5.44.105',
         },
         'glance/0': {
-            'broker_req': '{"api-version": 1, "request-id": "0bc7dc54", "ops": [{"replicas": 3, "name": "glance", "op": "create-pool"}]}',
+            'broker_req': '{"api-version": 1, "request-id": "0bc7dc54", "ops": [{"replicas": 3, "name": "glance", "op": "create-pool", "rbd-mirroring-mode": "pool"}]}',
             'private-address': '10.5.44.109',
         },
     }
@@ -267,6 +267,19 @@ class CephUtilsTests(TestCase):
             self._get_osd_settings_test_helper,
             settings)
 
+    @patch.object(ceph_utils, 'application_name')
+    def test_send_application_name(self, application_name):
+        application_name.return_value = 'client'
+        ceph_utils.send_application_name()
+        self.relation_set.assert_called_once_with(
+            relation_settings={'application-name': 'client'},
+            relation_id=None)
+        self.relation_set.reset_mock()
+        ceph_utils.send_application_name(relid='rid:1')
+        self.relation_set.assert_called_once_with(
+            relation_settings={'application-name': 'client'},
+            relation_id='rid:1')
+
     @patch.object(ceph_utils, 'get_osd_settings')
     def test_send_osd_settings(self, _get_osd_settings):
         self.relation_ids.return_value = ['client:1', 'client:3']
@@ -778,7 +791,7 @@ class CephUtilsTests(TestCase):
 
     @patch.object(ceph_utils, 'erasure_profile_exists')
     def test_create_erasure_profile(self, existing_profile):
-        existing_profile.return_value = True
+        existing_profile.return_value = False
         self.test_config.set('customize-failure-domain', False)
         self.cmp_pkgrevno.return_value = -1
         ceph_utils.create_erasure_profile(
@@ -788,8 +801,7 @@ class CephUtilsTests(TestCase):
 
         cmd = ['ceph', '--id', 'admin', 'osd', 'erasure-code-profile', 'set', 'super-profile',
                'plugin=' + 'jerasure', 'k=' + str(10), 'm=' + str(3),
-               'ruleset-failure-domain=' + 'rack',
-               '--force']
+               'ruleset-failure-domain=' + 'rack']
         self.check_call.assert_has_calls([call(cmd)])
 
         self.cmp_pkgrevno.return_value = 1
@@ -801,13 +813,22 @@ class CephUtilsTests(TestCase):
         cmd = ['ceph', '--id', 'admin', 'osd', 'erasure-code-profile', 'set', 'super-profile',
                'plugin=' + 'jerasure', 'k=' + str(10), 'm=' + str(3),
                'crush-failure-domain=' + 'rack',
-               'crush-device-class=ssd',
-               '--force']
+               'crush-device-class=ssd']
         self.check_call.assert_has_calls([call(cmd)])
 
+        existing_profile.return_value = True
+        self.check_call.reset_mock()
+
+        ceph_utils.create_erasure_profile(
+            service='admin', profile_name='super-profile', erasure_plugin_name='jerasure',
+            failure_domain='rack', data_chunks=10, coding_chunks=3,
+            device_class='ssd')
+
+        self.check_call.assert_not_called()
+
     @patch.object(ceph_utils, 'erasure_profile_exists')
     def test_create_erasure_profile_failure_domain(self, existing_profile):
-        existing_profile.return_value = True
+        existing_profile.return_value = False
         self.test_config.set('customize-failure-domain', True)
         self.cmp_pkgrevno.return_value = -1
         ceph_utils.create_erasure_profile(
@@ -817,8 +838,7 @@ class CephUtilsTests(TestCase):
 
         cmd = ['ceph', '--id', 'admin', 'osd', 'erasure-code-profile', 'set', 'super-profile',
                'plugin=' + 'jerasure', 'k=' + str(10), 'm=' + str(3),
-               'ruleset-failure-domain=' + 'rack',
-               '--force']
+               'ruleset-failure-domain=' + 'rack']
         self.config.assert_called_once_with('customize-failure-domain')
         self.check_call.assert_has_calls([call(cmd)])
 
@@ -1578,6 +1598,17 @@ class CephUtilsTests(TestCase):
         rq2.add_op_request_access_to_group(name='objects',
                                            permission='r')
         self.assertFalse(rq1 == rq2)
+        # now check equality check for common properties
+        rq1 = ceph_utils.CephBrokerRq()
+        rq1.add_op_create_replicated_pool('pool1')
+        rq2 = ceph_utils.CephBrokerRq()
+        rq2.add_op_create_replicated_pool('pool1')
+        self.assertTrue(rq1 == rq2)
+        rq1 = ceph_utils.CephBrokerRq()
+        rq1.add_op_create_replicated_pool('pool1', compression_mode='none')
+        rq2 = ceph_utils.CephBrokerRq()
+        rq2.add_op_create_replicated_pool('pool1', compression_mode='passive')
+        self.assertFalse(rq1 == rq2)
 
     def test_ceph_broker_rsp_class(self):
         rsp = ceph_utils.CephBrokerRsp(json.dumps({'exit-code': 0,
@@ -1976,6 +2007,7 @@ class CephUtilsTests(TestCase):
             'max-bytes': None,
             'max-objects': None,
             'group-namespace': None,
+            'rbd-mirroring-mode': 'pool',
             'weight': None,
         }
         self.assertDictEqual(
@@ -2009,6 +2041,7 @@ class CephUtilsTests(TestCase):
                    'name': 'apool',
                    'op': 'create-pool',
                    'pg_num': None,
+                   'rbd-mirroring-mode': 'pool',
                    'replicas': 3,
                    'weight': None}
         rq = ceph_utils.CephBrokerRq()
@@ -2102,6 +2135,16 @@ class CephUtilsTests(TestCase):
         op = base_op.copy()
         op['max-objects'] = 42
         self.assertEqual(rq.ops, [op])
+        rq = ceph_utils.CephBrokerRq()
+        rq.add_op_create_replicated_pool('apool', rbd_mirroring_mode='pool')
+        op = base_op.copy()
+        op['rbd-mirroring-mode'] = 'pool'
+        self.assertEqual(rq.ops, [op])
+        rq = ceph_utils.CephBrokerRq()
+        rq.add_op_create_replicated_pool('apool', rbd_mirroring_mode='image')
+        op = base_op.copy()
+        op['rbd-mirroring-mode'] = 'image'
+        self.assertEqual(rq.ops, [op])
 
     def test_add_op_create_erasure_pool(self):
         base_op = {'app-name': None,
@@ -2123,6 +2166,7 @@ class CephUtilsTests(TestCase):
                    'name': 'apool',
                    'op': 'create-pool',
                    'pool-type': 'erasure',
+                   'rbd-mirroring-mode': 'pool',
                    'weight': None}
         rq = ceph_utils.CephBrokerRq()
         rq.add_op_create_erasure_pool('apool')
